import { Client, Message, TextChannel } from 'discord.js';
import dedent from 'ts-dedent';
import { ImplementableApi } from './implementableApi';

import { once as eventsOnce } from 'events';

namespace DiscordParser {
    export type Config = ImplementableApi.Config & {
        token: string;
        channelsId: {
            idChannelYtb: string;
            idChannelPeerTube: string;
        };
        keyWord: string;
    };
}

class DiscordParser extends ImplementableApi {
    readonly token: string;
    readonly keyWord: string;
    readonly channels: Promise<{
        [key: string]: TextChannel;
        ChannelYtb: TextChannel;
        ChannelPeerTube: TextChannel;
    }>;
    readonly client: Client;

    constructor(readonly config: DiscordParser.Config) {
        super(config);
        this.token = config.token;
        this.keyWord = config.keyWord;

        this.client = this.instantiateClient();
        this.channels = eventsOnce(this.client, 'ready').then(() => {
            return this.setChannel(config.channelsId);
        });
        this.channels.then(() => {
            this.settingEvents();
        });
    }

    private setChannel(ids: {
        [key: string]: string;
        idChannelYtb: string;
        idChannelPeerTube: string;
    }): {
        ChannelYtb: TextChannel;
        ChannelPeerTube: TextChannel;
    } {
        let resp: {
            [key: string]: TextChannel | undefined;
            ChannelYtb?: TextChannel;
            ChannelPeerTube?: TextChannel;
        } = {};
        // construct and check the channels

        for (const key in ids) {
            if (ids[key]) {
                // console.log(ids[key]);
                const tmpChannel = this.client.channels.resolve(ids[key]);
                if (tmpChannel)
                    if (tmpChannel instanceof TextChannel)
                        resp[key.slice(2)] = tmpChannel;
                    else throw new Error('The channel must be a TextChannel');
                else
                    throw new Error(
                        dedent`The channel cannot be found by the bot. 
                Check if you had the bot to your server or if the channel ID is correct`
                    );
            }
        }
        // return the well formed object
        if (resp.ChannelPeerTube)
            if (resp.ChannelYtb)
                return {
                    ChannelPeerTube: resp.ChannelPeerTube,
                    ChannelYtb: resp.ChannelYtb,
                };

        console.log(resp);
        // console.log(resp.ChannelYtb);

        throw new Error('Theres an issue concerned the channel check');
    }

    private instantiateClient(): Client {
        const newClient = new Client();
        newClient.login(this.token);
        return newClient;
    }

    public receivedMessage(message: ImplementableApi.Message) {
        switch (message.type) {
            case 'newEntriesNotify':
                this.sendMsgYtb(
                    `New YouTubes entries received :\n${message.rawContent.items.map(
                        (item: any) =>
                            `Author : ${item.author}\nTitle: ${item.title}\nlink: ${item.link}`
                    )}`
                );
            default:
                break;
        }
    }

    private async sendMsgYtb(message: string) {
        const resolvedChannel = await this.channels;
        resolvedChannel.ChannelYtb.send(message);
    }
    private async sendMsgPeerTube(message: string) {
        const resolvedChannel = await this.channels;
        resolvedChannel.ChannelPeerTube.send(message);
    }

    private settingEvents(): void {
        this.on('message', async (message: Message) => {
            const resolvedChannel = await this.channels;
            let id_arr: string[] = [];
            for (const key in this.channels)
                id_arr.push(resolvedChannel[key].id);

            if (this.channels)
                if (id_arr.includes(message.channel.id)) {
                    const msg_splitted = message.content.split(' ');
                    if (msg_splitted[0] === this.keyWord) {
                        switch (msg_splitted[1]) {
                            case 'add':
                                const send_message: ImplementableApi.Message = {
                                    rawContent: {
                                        address: msg_splitted[2],
                                        user: message.author.toString(),
                                        date: message.createdAt.toUTCString(),
                                    },
                                    type: 'newListener',
                                };
                                message.channel.send('Ceci est un feedback');
                                this.emit('addListener', send_message);
                        }
                    }
                }
        });
    }
}

export { DiscordParser };
